#!/usr/bin/env bash

function check_status() {
    cli -m csv -c 'app chart_release query name,status' | grep $app | cut -d ',' -f 2 | tr -d '\n\r'
}

function usage() {
    cat <<-EOF
		Backup TrueCharts application to a Git repository.

		Usage: backup.tool [OPTION]...

		-h, --help             Display help
		-a, --app=NAME         Application name to backup
		-d, --dataset=DIR      Dataset directory name
		-p, --pool=DIR         Pool directory name
        -r, --replicas=NUMBER  Number of replicas, default 1
		-u, --user=NAME        User name to own the dataset directory, default root
		-s, --sync             Sync application data from current dataset directory
	EOF
    exit 0
}

if [ $EUID -gt 0 ]; then
    echo 'Run backup.tool as root.'
    exit 1
fi
if [ ! -x "$(command -v cli)" ]; then
    echo 'cli is not installed.'
    exit 1
fi
if [ ! -x "$(command -v git)" ]; then
    echo 'git is not installed.'
    exit 1
fi
if [ ! -x "$(command -v k3s)" ]; then
    echo 'k3s is not installed.'
    exit 1
fi
if [ ! -x "$(command -v rsync)" ]; then
    echo 'rsync is not installed.'
    exit 1
fi
if [ ! -x "$(command -v zfs)" ]; then
    echo 'zfs is not installed.'
    exit 1
fi

sync=false
options="$(getopt -l 'app:,dataset:,help,pool:,replicas:,sync,user:' -o 'a:d:hp:r:su:' -- "$@")"
eval set -- "$options"
while true; do
    case "$1" in
        -a|--app)
            app="$2"
            shift 2
            ;;
        -d|--dataset)
            dataset="$2"
            shift 2
            ;;
        -h|--help)
            usage
            ;;
        -p|--pool)
            pool="$2"
            shift 2
            ;;
        -r|--replicas)
            replicas="$2"
            shift 2
            ;;
        -s|--sync)
            sync=true
            shift
            ;;
        -u|--user)
            user="$2"
            shift 2
            ;;
        --)
            break
            ;;
        *)
            usage
            ;;
    esac
done
if [[ -z "$app" || -z "$dataset" || -z "$pool" ]]; then
    usage
fi
if [ ! -d /mnt/$pool ]; then
    echo 'Invalid pool directory name.'
    exit 1
fi
if [ ! -d /mnt/$pool/$dataset ]; then
    echo 'Invalid dataset directory name.'
    exit 1
fi
if [ -z "$replicas" ]; then
    replicas=1
fi
if [ -z "$user" ]; then
    user='root'
fi
if [ "$(id -u $user 2>/dev/null)" == "" ]; then
    echo 'Invalid user name.'
    exit 1
fi
volume="$(k3s kubectl get pvc -n ix-$app -o json | jq -r '.items[0].spec.volumeName')"
if [ "$volume" == "null" ]; then
    echo 'Invalid application name or not installed.'
    exit 1
fi

start=false
if [ "$(check_status)" == "ACTIVE" ]; then
    echo -n "Stopping $app "
    k3s kubectl scale deployments/$app -n ix-$app --replicas=0 >/dev/null 2>&1
    while true; do
        echo -n '.'
        sleep 1
        if [ "$(check_status)" == "STOPPED" ]; then
            break
        fi
    done
    echo ' OK'
    start=true
fi
if [ -d /mnt/$pool/$app ]; then
    rm -rf /mnt/$pool/$app
fi
name="$pool/ix-applications/releases/$app/volumes/$volume"
zfs set mountpoint=/$pool/$app $name
if $sync; then
    directory="/mnt/$pool/$app"
    rsync -av --delete /mnt/$pool/$dataset/$app/ $directory/
    push=false
else
    directory="/mnt/$pool/$dataset/$app"
    rsync -av --delete /mnt/$pool/$app/ $directory/
    push=true
fi
find $directory -type d -exec chmod a+rwx,g-w,o-w,ugo-s,-t {} +
find $directory -type f -name '.DS_Store' -delete
find $directory -type f -exec chmod a+rwx,u-x,g-wx,o-wx,ugo-s,-t {} +
chown -R $user:apps $directory
zfs set mountpoint=legacy $name
if $start; then
    echo -n "Starting $app "
    k3s kubectl scale deployments/$app -n ix-$app --replicas=$replicas >/dev/null 2>&1
    while true; do
        echo -n '.'
        sleep 1
        if [ "$(check_status)" == "ACTIVE" ]; then
            break
        fi
    done
    echo ' OK'
fi
rm -rf /mnt/$pool/$app
if [ -d /mnt/$pool/$dataset/.git ]; then
    pushd /mnt/$pool/$dataset >/dev/null 2>&1
    git pull --allow-unrelated-histories --depth=5 --no-edit -X theirs
    if [ "$?" -eq 0 ]; then
        if $push; then
            git add -A
            git commit -m "Updated $app files"
            git push
        fi
    fi
    popd >/dev/null 2>&1
fi
